package cn.com.ty.lift.ud.neety.task;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import cn.com.ty.lift.ud.chat.mapper.entity.ChatMsgEntity;
import cn.com.ty.lift.ud.chat.mapper.entity.ChatUnreadMsgEntity;
import cn.com.ty.lift.ud.chat.service.IChatMsgService;
import cn.com.ty.lift.ud.chat.service.IChatUnreadMsgService;
import cn.com.ty.lift.ud.neety.protocol.RoomVo;
import cn.com.ty.lift.ud.redis.ChatDo;
import cn.com.ty.lift.ud.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 同步缓存数据到mysql
 * 
 * @author sKing
 *
 */
@Slf4j
@Component
public class RedisTask {

    private @Autowired RedisUtil redisUtil;
    private @Autowired IChatUnreadMsgService chatUnreadMsgService;
    private @Autowired IChatMsgService msgService;

    private static RedisUtil redis;
    private static IChatUnreadMsgService unreadMsgService;
    private static IChatMsgService iChatMsgService;

    @PostConstruct
    public void initBean() {
        redis = redisUtil;
        unreadMsgService = chatUnreadMsgService;
        iChatMsgService = msgService;
    }

    public RedisTask() {
        log.info("RedisTask init");
    }

    /**
     * 每5分钟执行一次 同步redis缓存中未读消息的数据到mysql中
     */
//    @Scheduled(cron = "0 */5 * * * ?")
    public void updateMemberLevel() {
        // RedisUtil redis = ApplicationContextUtil.getBean(RedisUtil.class);
        // IChatUnreadMsgService unreadMsgService = ApplicationContextUtil.getBean(ChatUnreadMsgServiceImpl.class);
        log.info("执行保存未读消息操作！");
        List<ChatUnreadMsgEntity> unreadMsgList = new ArrayList<>();
        Map<String, List<RoomVo>> redisRoom = redis.get("ROOM", Map.class);
        ChatUnreadMsgEntity entity = new ChatUnreadMsgEntity();
        // 获取所有房间的聊天记录
        if (null != redisRoom) {

            for (String room : redisRoom.keySet()) {

                log.info("Start.  正在执行的房间号是：{}", room);
                String txt = redis.getKey("ROOM");
                Object ob = txt == null ? null : JSONObject.parseObject(txt, HashMap.class).get(room);
                if (null != ob) {
                    List<RoomVo> voList = JSONArray.parseArray(ob.toString(), RoomVo.class);

                    for (RoomVo item : voList) {

                        if (item.getMsgList().size() > 0) {
                            item.getMsgList().forEach(msg -> {
                                BeanUtils.copyProperties(msg, entity);
                                unreadMsgList.add(entity);
                            });

                            item.setMsgList(null);
                            item.setUnreadNum(0);

                        } else {

                            log.info("房间号是：{}----》暂无未读消息保存", room);
                        }

                    }
                    /** TODO 打开之后将清理redis中未读的缓存 暂时未用（使用之后用户上线查询未读消息需要结合缓存以及数据库的数据） */
                    // redisRoom.put(room, voList);
                }

            }
            // 重置缓存
            redis.setValue("ROOM", redisRoom);

            unreadMsgService.saveBatch(unreadMsgList);
        }
        log.info("End. 保存未读消息操作结束！");

    }

    /**
     * 每三分钟执行一次 同步redis缓存中聊天记录数据到mysql中 [秒] [分] [小时] [日] [月] [周] [年]
     */
//    @Scheduled(cron = "0 */3 * * * ?")
    // @Scheduled(fixedDelay = 60000 * 5)
    public void saveUnreadMsg() {
        // RedisUtil redis = ApplicationContextUtil.getBean(RedisUtil.class);
        // IChatMsgService msgService = ApplicationContextUtil.getBean(ChatMsgServiceImpl.class);
        log.info("执行保存聊天记录操作！");
        List<ChatMsgEntity> msg = new ArrayList<>();
        List<String> key = new ArrayList<>();// 创建临时房间列表。避免清除缓存的时候删除了新增的房间号
        Map<String, List<RoomVo>> redisRoom = redis.get("ROOM", Map.class);
        // 获取所有房间的聊天记录
        if (null != redisRoom) {
            for (String item : redisRoom.keySet()) {
                // logger.info("Start. 正在执行的房间号是：{}", item);
                ChatDo chatDo = redis.get(item, ChatDo.class);
                if (null != chatDo) {
                    chatDo.getList().forEach(chat -> {
                        ChatMsgEntity entity = new ChatMsgEntity();
                        BeanUtils.copyProperties(chat, entity);
                        LocalDateTime convertStr = LocalDateTime.parse(chat.getCreateTime());
                        entity.setCreateTime(convertStr);
                        msg.add(entity);
                    });
                    key.add(item);
                } else {
                    // logger.info("房间号是：{}----》暂无消息可保存", item);
                }
            }

            // 删除缓存中已提取的房间号
            redis.delete(key);

            iChatMsgService.saveBatch(msg);
        }
        log.info("End. 保存聊天记录操作结束！");
    }

}
